﻿using System;
using System.Linq;
using System.Collections.Generic;

/// @author  Vesa Lappalainen
/// @version 26.10.2013
/// <summary>
/// Tutkitaan listojen käyttäytymistä
/// </summary>
public class Listat
{
    /// <summary>
    /// Esimerkki listoista
    /// </summary>
    public static void Main()
    {
        int[] taulukko = { 3, 5, 6, 99, 5, 5, 2, 23 };
        int tkoko = taulukko.Length;
        Console.WriteLine(String.Join(" ",taulukko) + " koko: " + tkoko);
        taulukko[1] = 9;

        List<int> lista = new List<int>();
        lista.Add(3); lista.Add(2); lista.Add(5);
        Console.WriteLine(String.Join(" ", lista));
        lista[1] = 9;          // alkioihin viitataan kuten taulukossakin
        lista.Add(99);         // mutta voidaan lisätä uusiakin
        lista.Insert(2,55);    // jopa keskelle
        int lkoko = lista.Count; // Huom!  Määrä eri tavalla
        Console.WriteLine(String.Join(" ", lista) + " koko: " + lkoko);
        lista.Remove(99);      // ja poistaa (ensimmäisen)
        Console.WriteLine(String.Join(" ", lista) + " koko: " + lista.Count+ " " + lista.Capacity);

        int suurin = Suurin(lista);
        Console.WriteLine(suurin);

        // Lista voi olla mitä tyyppiä vaan. Listan voi alustaa saman tyypin taulukolla
        List<double> dluvut = new List<double>(){3.5,1.2,9.3,6.2};
        Console.WriteLine(String.Join(" ", dluvut));
        dluvut.Sort();  // listan voi järjestää jos on Linq käytössä
        Console.WriteLine(String.Join(" ", dluvut));

        double dsuurin = Suurin(dluvut);
        dsuurin = dluvut.Max(); // listan suurin löytyy näinkin jos on Linq käytössä
        Console.WriteLine(dsuurin);

        int isuurin = Suurin(taulukko);
        isuurin = taulukko.Max();
        Console.WriteLine(isuurin);

        // Erilaisia tapoja laskea montako 5:sta on listassa
        List<int> lista2 = new List<int>() { 3, 5, 6, 99, 5, 5, 2, 23 };
        int lkm5 = LaskeLuvut(lista2, 5);
        lkm5 = lista2.Sum(onko5); 
        lkm5 = lista2.Sum(delegate(int a) { if (a == 5) return 1; return 0; });
        lkm5 = lista2.Sum(a => a == 5 ? 1 : 0); // lambda lauseke
        int paikka5 = lista2.IndexOf(5);
        Console.WriteLine($"Viitosten lkm {lkm5}, ensimmäisen paikka {paikka5}");

        IEnumerable<int> vitoset = lista2.Where(a => a == 5);
        Console.WriteLine(String.Join(" ", vitoset));
        lkm5 = vitoset.Count();
        lkm5 = lista2.Where(a => a == 5).Count();
        lkm5 = lista2.Count(a => a == 5);
        Console.WriteLine($"Viitosten lkm {lkm5}, ensimmäisen paikka {paikka5}");

        Array.Sort(taulukko); // taulukon voi järjestää näin
        Console.WriteLine(String.Join(" ", taulukko) + " koko: " + tkoko);
        isuurin = taulukko.Max(); // taulukon/listan suurin löytyy näinkin jos on Linq käytössä
        isuurin = Enumerable.Max(taulukko); // tai näin
        Console.WriteLine(isuurin);
        int ipienin = -taulukko.Max(luku => -luku); // näin löytyisi pienin "erilaisella" tavalla
        Console.WriteLine(ipienin);
        int[] taulukko2 = { 3, 2, 5, 4 };
        Console.WriteLine(String.Join(" ", taulukko2));
        int suurinParillinen = taulukko2.Max(luku => luku % 2 == 0 ? luku : -100000);
        Console.WriteLine(String.Join(" ", taulukko2.Where(luku => luku % 2 == 0)));
        suurinParillinen = taulukko2.Where(luku => luku %2 == 0 ).Max();
        Console.WriteLine(suurinParillinen);

        dluvut.Sort((a,b) => -a.CompareTo(b));  // järjestetään päinvastoin
        Console.WriteLine(String.Join(" ", dluvut));
    }


    public static int onko5(int a)
    {
        if (a == 5) return 1;
        return 0;
    }


/*
    /// <summary>
    /// Etsitään listan suurin
    /// </summary>
    /// <param name="luvut">lista josta suurinta etsitään</param>
    /// <returns>listan suurin tai 0 jos lista tyhjä</returns>
    /// <example>
    /// <pre name="test">
    ///    List<int>lista = new List<int>(){5,3,8,7});
    ///    Suurin(lista) === 8;
    ///    Suurin(new List<int>(){1,2,3}) === 3;
    ///    Suurin(new List<int>(){4,2,3}) === 4;
    ///    Suurin(new List<int>(){1,5,3}) === 5;
    /// </pre>
    /// </example>
    public static int Suurin(List<int> luvut)
    {
        if (luvut.Count == 0) return 0;
        int suurin = luvut[0];
        foreach (int luku in luvut)
            if (luku > suurin) suurin = luku;
        return suurin;
    }


    /// <summary>
    /// Etsitään listan suurin
    /// </summary>
    /// <param name="luvut">lista josta suurinta etsitään</param>
    /// <returns>listan suurin tai 0 jos lista tyhjä</returns>
    /// <example>
    /// <pre name="test">
    ///    Suurin(new List<double>(){1,2,3}) ~~~ 3;
    ///    Suurin(new List<double>(){4,2,3}) ~~~ 4;
    ///    Suurin(new List<double>(){1,5,3}) ~~~ 5;
    /// </pre>
    /// </example>
    public static double Suurin(List<double> luvut)
    {
        if (luvut.Count == 0) return 0;
        double suurin = luvut[0];
        foreach (double luku in luvut)
            if (luku > suurin) suurin = luku;
        return suurin;
    }
*/


    /// <summary>
    /// Etsitään listan suurin
    /// </summary>
    /// <param name="luvut">lista josta suurinta etsitään</param>
    /// <returns>listan suurin</returns>
    /// <example>
    /// <pre name="test">
    ///    Suurin(new List<int>(){1,2,3}) === 3;
    ///    Suurin(new List<int>(){4,2,3}) === 4;
    ///    Suurin(new List<long>(){1,5,3}) === 5;
    ///    Suurin(new List<double>(){1,2,3}) ~~~ 3;
    ///    Suurin(new List<double>(){4,2,3}) ~~~ 4;
    ///    Suurin(new List<double>(){1,5,3}) ~~~ 5;
    ///    Suurin(new List<double>()) ~~~ 0;   #THROWS ArgumentOutOfRangeException
    ///    Suurin(new List<String>(){"kissa","auto","kettu"}) === "kissa";
    ///    Suurin(new double[]{1,5,3}) ~~~ 5;
    ///    Suurin(new int[]{4,2,3}) === 4;
    ///    Suurin(new String[]{"kissa","auto","kettu"}) === "kissa";
    /// </pre>
    /// </example>
    //    public static T Suurin<T>(List<T> luvut) where T : IComparable<T>
    public static T Suurin<T>(IList<T> luvut) where T : IComparable<T>
    {
        // if (luvut.Count == 0) return 0;
        T suurin = luvut[0]; 
        foreach (T luku in luvut)
            if (luku.CompareTo(suurin) > 0) suurin = luku;
        return suurin;
    }


    /// <summary>
    /// Etsitään taulukosta montako pyydytettyä lukua siellä on.
    /// </summary>
    /// <param name="luvut">taulukko josta etsitään</param>
    /// <param name="mitaEtsitaan">etsittävä luku</param>
    /// <returns>montako etsittävää löytui</returns>
    /// <example>
    /// <pre name="test">
    ///   int[] luvut = {1,2,3,4,3,2,0};
    ///   LaskeLuvut(luvut,5) === 0;
    ///   LaskeLuvut(luvut,3) === 2;
    ///   LaskeLuvut(luvut,1) === 1;
    ///   LaskeLuvut(new int[]{},1) === 0;
    /// </pre>
    /// </example>
    public static int LaskeLuvut(int[] luvut, int mitaEtsitaan)
    {
        int lkm = 0;
        /*
        for (int i = 0; i < luvut.Length; i++)
        {
            int luku = luvut[i];
            if (luku == mitaEtsitaan)
                lkm++;
        }
        */
        foreach (int luku in luvut)
            if (luku == mitaEtsitaan)
                lkm++;
        return lkm;
    }


    /// <summary>
    /// Etsitään listasta montako pyydytettyä lukua siellä on.
    /// </summary>
    /// <param name="luvut">lista josta etsitään</param>
    /// <param name="mitaEtsitaan">etsittävä luku</param>
    /// <returns>montako etsittävää löytui</returns>
    /// <example>
    /// <pre name="test">
    ///   List<int> luvut = new List<int>(){1,2,3,4,3,2,0};
    ///   LaskeLuvut(luvut,5) === 0;
    ///   LaskeLuvut(luvut,3) === 2;
    ///   LaskeLuvut(luvut,1) === 1;
    ///   LaskeLuvut(new List<int>(),1) === 0;
    /// </pre>
    /// </example>
    public static int LaskeLuvut<T>(List<T> luvut, T mitaEtsitaan)
    {
        int lkm = 0;
        foreach (T luku in luvut)
            if (luku.Equals(mitaEtsitaan))
                lkm++;
        return lkm;
    }
}
